@@ -40,6 +40,9 @@ import (
 	"github.com/dolthub/dolt/go/store/hash"
 )
 
+// Do not read more than 128MB at a time.
+const maxReadSize = 128 * 1024 * 1024 * 1024
+
 // CompressedChunk represents a chunk of data in a table file which is still compressed via snappy.
 type CompressedChunk struct {
 	// H is the hash of the chunk
@@ -672,11 +675,11 @@ var _ chunkReader = tableReader{}
 func (tr tableReader) readCompressedAtOffsets(
 	ctx context.Context,
 	rb readBatch,
-	found func(CompressedChunk),
+	found func(context.Context, CompressedChunk),
 	stats *Stats,
 ) error {
-	return tr.readAtOffsetsWithCB(ctx, rb, stats, func(cmp CompressedChunk) error {
-		found(cmp)
+	return tr.readAtOffsetsWithCB(ctx, rb, stats, func(ctx context.Context, cmp CompressedChunk) error {
+		found(ctx, cmp)
 		return nil
 	})
 }
@@ -684,17 +687,17 @@ func (tr tableReader) readCompressedAtOffsets(
 func (tr tableReader) readAtOffsets(
 	ctx context.Context,
 	rb readBatch,
-	found func(*chunks.Chunk),
+	found func(context.Context, *chunks.Chunk),
 	stats *Stats,
 ) error {
-	return tr.readAtOffsetsWithCB(ctx, rb, stats, func(cmp CompressedChunk) error {
+	return tr.readAtOffsetsWithCB(ctx, rb, stats, func(ctx context.Context, cmp CompressedChunk) error {
 		chk, err := cmp.ToChunk()
 
 		if err != nil {
 			return err
 		}
 
-		found(&chk)
+		found(ctx, &chk)
 		return nil
 	})
 }
@@ -703,7 +706,7 @@ func (tr tableReader) readAtOffsetsWithCB(
 	ctx context.Context,
 	rb readBatch,
 	stats *Stats,
-	cb func(cmp CompressedChunk) error,
+	cb func(ctx context.Context, cmp CompressedChunk) error,
 ) error {
 	readLength := rb.End() - rb.Start()
 	buff := make([]byte, readLength)
@@ -723,7 +726,7 @@ func (tr tableReader) readAtOffsetsWithCB(
 			return err
 		}
 
-		err = cb(cmp)
+		err = cb(ctx, cmp)
 		if err != nil {
 			return err
 		}
@@ -738,7 +741,7 @@ func (tr tableReader) getMany(
 	ctx context.Context,
 	eg *errgroup.Group,
 	reqs []getRecord,
-	found func(*chunks.Chunk),
+	found func(context.Context, *chunks.Chunk),
 	stats *Stats) (bool, error) {
 
 	// Pass #1: Iterate over |reqs| and |tr.prefixes| (both sorted by address) and build the set
@@ -747,7 +750,7 @@ func (tr tableReader) getMany(
 	err := tr.getManyAtOffsets(ctx, eg, offsetRecords, found, stats)
 	return remaining, err
 }
-func (tr tableReader) getManyCompressed(ctx context.Context, eg *errgroup.Group, reqs []getRecord, found func(CompressedChunk), stats *Stats) (bool, error) {
+func (tr tableReader) getManyCompressed(ctx context.Context, eg *errgroup.Group, reqs []getRecord, found func(context.Context, CompressedChunk), stats *Stats) (bool, error) {
 	// Pass #1: Iterate over |reqs| and |tr.prefixes| (both sorted by address) and build the set
 	// of table locations which must be read in order to satisfy the getMany operation.
 	offsetRecords, remaining := tr.findOffsets(reqs)
@@ -755,7 +758,7 @@ func (tr tableReader) getManyCompressed(ctx context.Context, eg *errgroup.Group,
 	return remaining, err
 }
 
-func (tr tableReader) getManyCompressedAtOffsets(ctx context.Context, eg *errgroup.Group, offsetRecords offsetRecSlice, found func(CompressedChunk), stats *Stats) error {
+func (tr tableReader) getManyCompressedAtOffsets(ctx context.Context, eg *errgroup.Group, offsetRecords offsetRecSlice, found func(context.Context, CompressedChunk), stats *Stats) error {
 	return tr.getManyAtOffsetsWithReadFunc(ctx, eg, offsetRecords, stats, func(
 		ctx context.Context,
 		rb readBatch,
@@ -768,7 +771,7 @@ func (tr tableReader) getManyAtOffsets(
 	ctx context.Context,
 	eg *errgroup.Group,
 	offsetRecords offsetRecSlice,
-	found func(*chunks.Chunk),
+	found func(context.Context, *chunks.Chunk),
 	stats *Stats,
 ) error {
 	return tr.getManyAtOffsetsWithReadFunc(ctx, eg, offsetRecords, stats, func(
@@ -807,7 +810,7 @@ func toReadBatches(offsets offsetRecSlice, blockSize uint64) []readBatch {
 			continue
 		}
 
-		if _, canRead := canReadAhead(rec, batch.End(), blockSize); canRead {
+		if _, canRead := canReadAhead(rec, batch.Start(), batch.End(), blockSize); canRead {
 			batch = append(batch, rec)
 			i++
 			continue
@@ -906,18 +909,22 @@ func (tr tableReader) findOffsets(reqs []getRecord) (ors offsetRecSlice, remaini
 	return ors, remaining
 }
 
-func canReadAhead(fRec offsetRec, readEnd, blockSize uint64) (newEnd uint64, canRead bool) {
-	if fRec.offset < readEnd {
+func canReadAhead(fRec offsetRec, curStart, curEnd, blockSize uint64) (newEnd uint64, canRead bool) {
+	if fRec.offset < curEnd {
 		// |offsetRecords| will contain an offsetRecord for *every* chunkRecord whose address
 		// prefix matches the prefix of a requested address. If the set of requests contains
 		// addresses which share a common prefix, then it's possible for multiple offsetRecords
 		// to reference the same table offset position. In that case, we'll see sequential
 		// offsetRecords with the same fRec.offset.
-		return readEnd, true
+		return curEnd, true
 	}
 
-	if fRec.offset-readEnd > blockSize {
-		return readEnd, false
+	if curEnd-curStart >= maxReadSize {
+		return curEnd, false
+	}
+
+	if fRec.offset-curEnd > blockSize {
+		return curEnd, false
 	}
 
 	return fRec.offset + uint64(fRec.length), true
@@ -949,7 +956,7 @@ func (tr tableReader) calcReads(reqs []getRecord, blockSize uint64) (reads int,
 			continue
 		}
 
-		if newReadEnd, canRead := canReadAhead(rec, readEnd, tr.blockSize); canRead {
+		if newReadEnd, canRead := canReadAhead(rec, readStart, readEnd, tr.blockSize); canRead {
 			readEnd = newReadEnd
 			i++
 			continue
